/*******************************************************************************

  Caixinha de Musica Eletrnica

  C/ Pic 12F675 - Arquivos RTTTL

  Autor: Cludio Lrios

  Data: 25/09/2016


  Use compilador CCS C

  Material Didtico

  Este arquivo  parte integrante do blog Lrios.tecnologia.ws

*******************************************************************************/

#include <12F675.h>
#device adc=8
#use delay(clock=20000000)
#fuses NOWDT,HS, NOCPD, NOPROTECT, NOMCLR, NOPUT, NOBROWNOUT


#bit  out1  =    0x5.2
#bit  out2  =    0x5.1
//#bit  sw    =    0x5.0
#bit  t0ie  =    0xb.5
#bit  t0if  =    0xb.2
#bit  tmr1ie  =    0x8c.0
#bit  tmr1if  =    0xc.0
#bit  peie    =  0x8b.6
#bit  gie     =  0x8b.7

#bit  pullup0 =  0x95.0

#byte  trisio =  0x85
#byte  gpio  =   0x05
#byte  option_reg = 0x81
#byte  t1con =   0x10
#byte  tmr0    =  0x01
#byte  tmr1h    =  0x0f
#byte  tmr1l    =  0x0e
#byte  intcon   =  0x0b



unsigned int1 f_som;//flag indicador de nota sendo reproduzida
unsigned int1 f_dur;//flag da durao adicional
unsigned int8 vel_ritmo;
unsigned int8 pont;
unsigned int8 oitava;
unsigned int8 duracao;
unsigned int16 nota;
unsigned int8 def_oitava ;
unsigned int8 def_duracao;
unsigned int8 cont;
unsigned int8 carga_tmr1h;
unsigned int8 carga_tmr1l;





/******************************************************************************
                 Rotinas de Interrupes dos Timers
*******************************************************************************/

#INT_TIMER0
void isr_tmr0(){
   tmr0 = vel_ritmo;
   if (cont) cont--;
  // t0if = 0;
  }
#INT_TIMER1
void isr_tmr1(){
   if (f_som) { out2=out1; out1 = !out1; }//produz som na sada
   else { out1 = 0; out2=0;}//desliga som na sada
   tmr1h = carga_tmr1h;
   tmr1l = carga_tmr1l;
 //  tmr1if = 0; //apaga flag do overflow tmr1
}


/******************************************************************************

                      Funes

*******************************************************************************/

/******************************************************************************
           Reproduz nota musical
*******************************************************************************/
void Reproduz_Nota(unsigned int16 not, unsigned int8 oit, unsigned int8 dur)
{
    switch (oit){
        case 5 : {//divide por 2
             not = not >> 1;
             break;}
        case 6 :{ //divide por 4
             not = not >> 2;
             break;}
        case 7 :{ //divide por 8
             not = not >> 4;
             break;}
    }

    while(cont) ;//aguarda reproduzir a nota musical selecionada anteriormente
    f_som = 0;
    
    if (not) {//se no for '0'
        not = ~not;//complementa
        carga_tmr1l = not;//carrega timer 1 l
        carga_tmr1h = not >> 8;//carrega timer 1 h
        f_som = 1;//indica que ir reproduzir uma nota
    }
    
    cont = 255/dur;//carrega valor da durao da nota ou pausa
    if(f_dur) cont+= cont>>1;//soma 1/2 a durao

}




/*******************************************************************************

       Rotina Principal c/ Inicializao de ports e registradores

******************************************************************************/

void main() {
#define musica1 //escolha a musica a ser tocada e recompile
//#define unica_reproducao  //comente para reproduzir continuamente

#ifdef musica1
//Beethoven - Fur Elise : d=4,o=5,b=140:8e6,8d#6,8e6,8d#6,8e6,8b,8d6,8c6,a,8p,8c,8e,8a,b,8p,8e,8g#,8b,c6,p,8e,8e6,8d#6,8e6,8d#6,8e6,8b,8d6,8c6,a,8p,8c,8e,8a,b,8p,8e,8c6,8b,2a,
const unsigned char musica[]= {"8e6,8d#6,8e6,8d#6,8e6,8b,8d6,8c6,a,8p,8c,8e,8a,b,8p,8e,8g#,8b,c6,p,8e,8e6,8d#6,8e6,8d#6,8e6,8b,8d6,8c6,a,8p,8c,8e,8a,b,8p,8e,8c6,8b,2a,1p,1p,1p@@"};
   def_oitava = 5;//valor default da oitava 'o=5'
   def_duracao = 4;//valor default da durao 'd=4'
   vel_ritmo = 140;//valor default da velocidade de execuo 'b=140'
#endif

#ifdef musica2
const unsigned char musica[]= {"c,e,c,g,c,c6,8b,8a,8g,8a,8g,8f,8e,8f,8e,8d,c,e,g,e,c6,g@@"};
  //  Beethoven:d=4,o=5,b=160:c,e,c,g,c,c6,8b,8a,8g,8a,8g,8f,8e,8f,8e,8d,c,e,g,e,c6,g
   def_oitava = 5;//valor default da oitava
   def_duracao = 4;//valor default da durao
   vel_ritmo = 160;//valor default da velocidade de execuo
 #endif



#ifdef musica3
//Mission:d=4,o=6,b=100:32d,32d#,32d,32d#,32d,32d#,32d,32d#,32d,32d,32d#,32e,32f,32f#,32g,16g,8p,16g,8p,16a#,16p,16c,16p,16g,8p,16g,8p,16f,16p,16f#,16p,16g,8p,16g,8p,16a#,16p,16c,16p,16g,8p,16g,8p,16f,16p,16f#,16p,16a#,16g,2d,32p,16a#,16g,2c#,32p,16a#,16g,2c,16p,16a#5,16c
  const unsigned char musica[]= {"32d,32d#,32d,32d#,32d,32d#,32d,32d#,32d,32d,32d#,32e,32f,32f#,32g,16g,8p,16g,8p,16a#,16p,16c,16p,16g,8p,16g,8p,16f,16p,16f#,16p,16g,8p,16g,8p,16a#,16p,16c,16p,16g,8p,16g,8p,16f,16p,16f#,16p,16a#,16g,2d,32p,16a#,16g,2c#,32p,16a#,16g,2c,16p,16a#5,16c@@"};
  def_oitava = 6;//valor default da oitava
  def_duracao = 4;//valor default da durao
  vel_ritmo = 100;//valor default da velocidade de execuo
#endif


#ifdef musica4
const unsigned char musica[]= {"16g,16g,16g,16d#,16g#,16g#,16g#,16g,16d#6,16d#6,16d#6,c6,16g,16g,16g,16d,16g#,16g#,16g#,16g,16f6,16f6,16f6,d6,16g6,16g6,16f6,16d#6,16d#,16d#,16f,16g,16g6,16g6,16f6,16d#6,16d#,16d#,16f,16g,16g6,16g6,16f6,8d#6,p,8c6,p,2g6@@"};
//Beethoven - 5th Symphony : d=4,o=5,b=125:16g,16g,16g,16d#,16g#,16g#,16g#,16g,16d#6,16d#6,16d#6,c6,16g,16g,16g,16d,16g#,16g#,16g#,16g,16f6,16f6,16f6,d6,16g6,16g6,16f6,16d#6,16d#,16d#,16f,16g,16g6,16g6,16f6,16d#6,16d#,16d#,16f,16g,16g6,16g6,16f6,8d#6,p,8c6,p,2g6,
   def_oitava = 5;//valor default da oitava
   def_duracao = 4;//valor default da durao
   vel_ritmo = 125;//valor default da velocidade de execuo
 #endif
 

#ifdef musica5
  const unsigned char musica[]= {"16f#5,16e5,16d#5,16e5,g5,16a5,16g5,16f#5,16g5,b5,16c,16b5,16a#5,16b5,16f#,16e,16d#,16e,16f#,16e,16d#,16e,g,8e,8g,32d,32e,16f#,8e,8d,8e,32d,32e,16f#,8e,8d,8e,32d,32e,16f#,8e,8d,8c#,b5,b5@@"};
//Mozart:d=4,o=6,b=120:16f_5,16e5,16d_5,16e5,g5,16a5,16g5,16f_5,16g5,b5,16c,16b5,16a_5,16b5,16f_,16e,16d_,16e,16f_,16e,16d_,16e,g,8e,8g,32d,32e,16f_,8e,8d,8e,32d,32e,16f_,8e,8d,8e,32d,32e,16f_,8e,8d,8c_,b5

   def_duracao = 4;//valor default da durao
   def_oitava = 6;//valor default da oitava
   vel_ritmo = 120;//valor default da velocidade de execuo
#endif


#ifdef musica6
const unsigned char musica[]= {"32f#.5,g5,32a.5,a#5,32f#.5,16g.5,32a.5,16a#.5,32d#.,16d.,32g.5,16a#.5,32d.,2c#,32c.,32a#.5,32g.5,32f.5,g.5,32p,16g.,32f.,16d.,32c.,16a#.5,32g.5,32c#,8c,32c#,8c,32c#,8c,32c#,8c,32a#.5,32g.5,32f.5,16g.5,g.5@@"};
//Pink:d=4,o=6,b=70:32f#.5,g5,32a.5,a#5,32f#.5,16g.5,32a.5,16a#.5,32d#.,16d.,32g.5,16a#.5,32d.,2c#,32c.,32a#.5,32g.5,32f.5,g.5,32p,16g.,32f.,16d.,32c.,16a#.5,32g.5,32c#,8c,32c#,8c,32c#,8c,32c#,8c,32a#.5,32g.5,32f.5,16g.5,g.5
   def_duracao = 4;//valor default da durao
   def_oitava = 6;//valor default da oitava
   vel_ritmo = 70;//valor default da velocidade de execuo
#endif

#ifdef musica7
//6Million:d=16,o=6,b=80:c#.,f#.,b.,4g#.,a.,g#.,e.,d.,b.5,c#.,e.,d#.,2c#,p,4b5,c#.,f#.,b.,4g#.,a.,g#.,e.,d.,b.5,c#.,e.,d#.,2c#,32f#5,32g#5,32a5,32b5,32c#,32d,32e,32f,4f#,c#,p,b5,p,e5,4f#5
const unsigned char musica[]= {"c#.,f#.,b.,4g#.,a.,g#.,e.,d.,b.5,c#.,e.,d#.,2c#,p,4b5,c#.,f#.,b.,4g#.,a.,g#.,e.,d.,b.5,c#.,e.,d#.,2c#,32f#5,32g#5,32a5,32b5,32c#,32d,32e,32f,4f#,c#,p,b5,p,e5,4f#5@@"};
   def_duracao = 16;//valor default da durao
   def_oitava = 6;//valor default da oitava
   vel_ritmo = 80;//valor default da velocidade de execuo
 #endif   
   
   
#ifdef musica8
//AdamsFamTheme:d=4,o=6,b=70:8f_5,16f_5,16f_5,16f_5,8b5,32d_,8b5,32g_5,8e5,8c_,32a5,8a_5,32c_,8a_5,32f_5,8d_5,8b5,32f_5,8b5,32d_,8b5,32g_5,8e5,8c_,32b5,8a_5,32f_5,8g_5,32a_5,b5,32f_5,8b5,32d_,8b5,32g_5,8e5,8c_,32a5,8a_5,32c_,8a_5,32f_5,8d_5,8b5,32f_5,8b5,32d_,8b5,32g_5,8e5
const unsigned char musica[]= {"8f#5,16f#5,16f#5,16f#5,8b5,32d#,8b5,32g#5,8e5,8c#,32a5,8a#5,32c#,8a#5,32f#5,8d#5,8b5,32f#5,8b5,32d#,8b5,32g#5,8e5,8c#,32b5,8a#5,32f#5,8g#5,32a#5,b5,32f#5,8b5,32d#,8b5,32g#5,8e5,8c#,32a5,8a#5,32c#,8a#5,32f#5,8d#5,8b5,32f#5,8b5,32d#,8b5,32g#5,8e5@@"};
   def_duracao = 4;//valor default da durao
   def_oitava = 6;//valor default da oitava
   vel_ritmo = 70;//valor default da velocidade de execuo
#endif


#ifdef musica9
//AdamsFamThemeOrig:d=4,o=6,b=160:8c,f,8a,f,8c,b5,2g,8f,e,8g,e,8e5,a5,2f,8c,f,8a,f,8c,b5,2g,8f,e,8c,d,8e,1f,8c,8d,8e,8f,1p,8d,8e,8f_,8g,1p,8d,8e,8f_,8g,p,8d,8e,8f_,8g,p,c,8e,1f
const unsigned char musica[]= {"8c,f,8a,f,8c,b5,2g,8f,e,8g,e,8e5,a5,2f,8c,f,8a,f,8c,b5,2g,8f,e,8c,d,8e,1f,8c,8d,8e,8f,1p,8d,8e,8f#,8g,1p,8d,8e,8f#,8g,p,8d,8e,8f#,8g,p,c,8e,1f@@"};
   def_duracao = 4;//valor default da durao
   def_oitava = 6;//valor default da oitava
   vel_ritmo = 160;//valor default da velocidade de execuo
 #endif  
   

#ifdef musica10
//BachMinuet:d=8,o=5,b=125:4d6,g,a,b,c6,4d6,4g,4g,4e6,c6,d6,e6,f6,4g6,4g,4g,4c6,d6,c6,b,a,4b,c6,b,a,g,4f#,g,a,b,g,4b,4a
const unsigned char musica[]= {"4d6,g,a,b,c6,4d6,4g,4g,4e6,c6,d6,e6,f6,4g6,4g,4g,4c6,d6,c6,b,a,4b,c6,b,a,g,4f#,g,a,b,g,4b,4a@@"};
  def_duracao = 8;//valor default da durao
   def_oitava = 5;//valor default da oitava
   vel_ritmo = 125;//valor default da velocidade de execuo
#endif


#ifdef musica11
//Badineri:d=16,o=6,b=125:8b,d7,b,8f#,b,f#,8d,f#,d,4b5,f#5,b5,d,b5,c#,b5,c#,b5,a#5,c#,e,c#,8d,8b5,8b,d7,b,8f#,b,f#,8d,f#,d,4b5,8d,8d,8d,8d,8b,8d,32d,32c#,32d,32c#,8c#,8f#,8f#,8f#,8f#,8d7,8f#,32f#,32f,32f#,32f,8f,c#,f#,a,f#,g#,f#,g#,f#,f,g#,b,g#,a,g#,a,g#,f#,a,f#,f,f#,b,f#,f,f#,c#7,f#,f,f#,d7,f#,f,f#,d7,c#7,b,c#7,a,g#,f#,8a,8g#,4f#
const unsigned char musica[]= {"8b,d7,b,8f#,b,f#,8d,f#,d,4b5,f#5,b5,d,b5,c#,b5,c#,b5,a#5,c#,e,c#,8d,8b5,8b,d7,b,8f#,b,f#,8d,f#,d,4b5,8d,8d,8d,8d,8b,8d,32d,32c#,32d,32c#,8c#,8f#,8f#,8f#,8f#,8d7,8f#,32f#,32f,32f#,32f,8f,c#,f#,a,f#,g#,f#,g#,f#,f,g#,b,g#,a,g#,a,g#,f#,a,f#,f,f#,b,f#,f,f#,c#7,f#,f,f#,d7,f#,f,f#,d7,c#7,b,c#7,a,g#,f#,8a,8g#,4f#@@"};
  def_duracao = 16;//valor default da durao
   def_oitava = 6;//valor default da oitava
   vel_ritmo = 125;//valor default da velocidade de execuo
#endif


#ifdef musica12
//Bourree:d=8,o=6,b=140:e,f#,4g,32p,16g,32g,32f#,e,4d#,e,f#,4b5,c#,d#,4e,d,c,4b5,a5,g5,4f#5,g5,a5,b5,a5,g5,f#5,4e5,32p,e,f#,4g,32p,16g,32g,32f#,e,4d#,e,f#,4b5,c#,d#,4e,d,c,4b5,a5,g5,32f#5,32e5,32f#5,4f#5,32p,g5,2g.5
const unsigned char musica[]= {"e,f#,4g,32p,16g,32g,32f#,e,4d#,e,f#,4b5,c#,d#,4e,d,c,4b5,a5,g5,4f#5,g5,a5,b5,a5,g5,f#5,4e5,32p,e,f#,4g,32p,16g,32g,32f#,e,4d#,e,f#,4b5,c#,d#,4e,d,c,4b5,a5,g5,32f#5,32e5,32f#5,4f#5,32p,g5,2g.5@@"};
  def_duracao = 8;//valor default da durao
   def_oitava = 6;//valor default da oitava
   vel_ritmo = 140;//valor default da velocidade de execuo
#endif

   setup_adc_ports(NO_ANALOGS|VSS_VDD);
   setup_adc(ADC_OFF);
   setup_comparator(NC_NC_NC_NC);
   gpio=0;
   trisio=111001;
   option_reg=0b01010111;;//liga pull up no pino 7 (sw);tmr0/256,internal
   t1con = 0b00000001;   //liga timer 1 prescaler/1
   tmr1if = 0;          //apaga flag de overflow
   tmr1ie = 1;         //liga interrupo do timer 1
   intcon= 0b11100000; //liga interrupes


/*******************************************************************************

           Loop  de repetio

*******************************************************************************/


 for(;;){

    f_dur=0;//default para tempo normal
    f_som=0;//desliga sadas de som
    pont=0;//zera ponteiro de notas
    cont=0;//zera contador de tempo da durao das notas


   do {
        oitava = def_oitava; 	//ajusta oitava inicialmente como default
        duracao = def_duracao;//ajusta durao inicialmente como default
        nota=0;//ajusta para pausa como default

        if ((musica[pont] == '3') && (musica[pont+1] == '2')) {
            duracao=32;
            pont+=2;
        }
        else if ((musica[pont] == '1') && (musica[pont+1] == '6')) {
            duracao=16;
            pont+=2;
        }
        else if (musica[pont] == '8') {
            duracao=8;
            pont++;
        }
        else if (musica[pont] == '4') {
            duracao = 4;
            pont++;
        }
        else if (musica[pont] == '2') {
            duracao=2;
            pont++;
        }
        else if (musica[pont] == '1') {
            duracao=1;
            pont++;
        }
        if (musica[pont + 1] == '#') {
            //alterao p/ nota sustenida
            switch (musica[pont]) {
                case 'a': {nota = 0x29E6;
                           break;}
                case 'c': {nota = 0x233B;
                           break;}
                case 'd': {nota = 0x1f63;
                           break;}
                case 'f': {nota = 0x1a65;
                           break;}
                case 'g': {nota = 0x1788;
                           break;}
                }
            pont+=2;
            }
            
            else {
            switch (musica[pont]) {
                case 'a': {nota = 0x2c64;
                           break;}
                case 'b':{ nota = 0x278B;
                           break;}
                case 'c': {nota = 0x2553;
                           break;}
                case 'd': {nota = 0x2141;
                           break;}
                case 'e': {nota = 0x1DA0;
                           break;}
                case 'f': {nota = 0x1BF6;
                           break;}
                case 'g': {nota = 0x18EA;
                           break;}
            }
            pont++;
        }

        if (musica[pont] == '.') {
            //aumenta a durao em mais 1/2
            f_dur=1;
            pont++;
        }
        if (musica[pont] == '4') {
            oitava=4;
            pont++;
        } else if (musica[pont] == '5') {
            oitava=5;
            pont++;
        } else if (musica[pont] == '6') {
            oitava=6;
            pont++;
        } else if (musica[pont] == '7') {
            oitava=7;
            pont++;
        }
        if (musica[pont] == '.') {
            //aumenta a durao em mais 1/2
            f_dur=1;
            pont++;
        }
        Reproduz_Nota(nota, oitava, duracao);//reproduz a nota

      } while (musica[pont++] != '@');//aguarda o termino da musica
     #ifdef unica_reproducao  //se for para executar uma nica vez
     f_som=0;
     for(;;);//reproduz uma unica vez
     #endif
 } //for(;;)
}//main()
